#!/usr/bin/perl

# Bulk Order from Emails
#use locale;
use POSIX qw(locale_h);
use DBI;

do "common.cfg";

$db = DBI->connect("dbi:Pg:dbname=$dbname") or die("Unable to connect to $dbname");
# Configurable data
$formfile = "templates/conform.ps";           # Danish template file
$outfile = ">invoices/invoice#.ps";

  my ($date, $debtac_id, $salesac_id, $vatac_id, $all_template) = init();
  my ($invh, $glbatchnr, $invbatchnr) = beginBatch();
  $outfile =~ s/#/$invbatchnr/e;
  open OUT, $outfile or die("Can't open $outfile\n");
  open IN, $formfile or die("Can't open $formfile\n");
  while (<IN>) {print OUT};
  close IN;
  processBatch($date, $debtac_id, $salesac_id, $vatac_id, $all_template,
               $invh, $glbatchnr, $invbatchnr);
 close OUT;

# ------ Init ------
sub init {
 my $date = FormatDate();
 $q = "SELECT coa_id FROM gl_chartofaccount WHERE number = $debacctnr";
 $sth = $db->prepare($q);
 $sth->execute or die("Can't execute $q\n");
 my ($debtac_id) = $sth->fetchrow();
 $q = "SELECT coa_id FROM gl_chartofaccount WHERE number = $salesacctnr";
 $sth = $db->prepare($q);
 $sth->execute or die("Can't execute $q\n");
 my ($salesac_id) = $sth->fetchrow();
 $q = "SELECT coa_id FROM gl_chartofaccount WHERE number = $vatacctnr";
 $sth = $db->prepare($q);
 $sth->execute or die("Can't execute $q\n");
 my ($vatac_id) = $sth->fetchrow();
 $q = "SELECT template FROM so_invoicetemplate i, so_invtplchoice c WHERE c.ordernr IS NULL
  AND c.contact_id IS NULL AND c.custgrp_id IS NULL AND c.invtplnr = i.invtplnr;";
 $sth = $db->prepare($q);
 $sth->execute or die("Can't execute $q\n");
 my ($all_template) = $sth->fetchrow();
 die('Invoicetemplate not found\n') if !$all_template;
 $sth->finish;
 return ($date, $debtac_id, $salesac_id, $vatac_id, $all_template);
};

sub beginBatch {
# setlocale(LC_NUMERIC, "da_DK");
 my $invbatchnr = $ARGV[0];
 help() if !$invbatchnr;
 my $q = "SELECT i.*, o.contact_id, o.paycode, o.yourref, c.name, c.custgrp_id, custtype, c.address1 as address, c.postalcode,
  c.city, date_trunc('month',invoicedate) AS periodnr
  FROM so_invoice i, so_orderhead o, co_custaddr_v c, so_invoicebatch b
  WHERE b.batchnr = $invbatchnr AND b.status = 100 AND i.status = 100
  AND i.batchnr = b.batchnr AND i.ordernr = o.ordernr AND o.contact_id = c.contact_id
  ORDER BY invoicenr";
 my $invh = $db->prepare($q);
 $invh->execute;
 exit 2 if !$invh->rows;

# DB start
 $db->{AutoCommit} = 0;
# GL start
 $q = "INSERT INTO gl_batch (system_id, type, batchnr, name, status, createddate, posteddate)
  VALUES (10001, 120001, $invbatchnr, 'Faktureringskrsel '||$invbatchnr, 200, 'now', 'now') ";
 my $sth = $db->do($q);
 $q = "SELECT currval('gl_batchsequence')";
 $sth = $db->prepare($q);
 $sth->execute;
 die("Can't execute $q\n") if !$sth->rows;
 my ($glbatchnr) = $sth->fetchrow();

# PS start
 return ($invh, $glbatchnr, $invbatchnr);
};

sub processBatch {
 ($date, $debtac_id, $salesac_id, $vatac_id, $all_template,
  $invh, $glbatchnr, $invbatchnr) = @_;
 my $cur_tpl;
 my ($batchdr, $batchcr, $journalnr);
# ----- Invoice loop -----
 while ( my $invoice = $invh->fetchrow_hashref ) {
  my $periodnr = $invoice->{periodnr};
  my ($attention, $payterms);
# SO update
  my $q = "UPDATE so_invoice SET status = 200, updated='now' WHERE invoicenr = $invoice->{invoicenr}";
  my $sth = $db->do($q);
  $q = "UPDATE so_orderhead SET status = 200, updated='now' WHERE ordernr = $invoice->{ordernr}";
  $sth = $db->do($q);
  $q = "UPDATE so_invoicebatch SET ordernr = $invoice->{ordernr}, status = 200, updated='now' WHERE batchnr = $invbatchnr";
  $sth = $db->do($q);
# GL update
  my ($custac_id) = 0;
  if ($invoice->{custgrp_id}) {
   $custac_id = $invoice->{custgrp_id};
#   $q = "SELECT ac_id FROM co_custgrp WHERE custgrp_id = $invoice->{custgrp_id}";
#   $sth = $db->prepare($q);
#   $sth->execute;
#   ($custac_id) = $sth->fetchrow()
  };
# Find template
 my $tpl = findTemplate($invoice->{ordernr}, $invoice->{contact_id}, $invoice->{custgrp_id});
 if ($tpl ne $cur_tpl) {
  my $compfile = "templates/$tpl.ps";
  open IN, $compfile or die("Can't open $compfile\n");
  while (<IN>) {print OUT};
  close IN;
  $cur_tpl = $tpl;
 }
# PS Stuff / get data
   $q = "SELECT name FROM co_paymentterms WHERE paycode = $invoice->{paycode}";
   $sth = $db->prepare($q);
   $sth->execute;
   ($payterms) = $sth->fetchrow() if $sth->rows;
  if ($invoice->{custtype} == 60002) {
   $q = "SELECT attention FROM co_company WHERE contact_id = $invoice->{contact_id}";
   $sth = $db->prepare($q);
   $sth->execute;
   next if !$sth->rows;
   ($attention) = $sth->fetchrow();
  };
  $q = "SELECT * FROM so_orderline WHERE ordernr = $invoice->{ordernr}";
  my $ordl = $db->prepare($q);
  $ordl->execute;
  next if !$ordl->rows;
# PS Stuff / header
  my $copies = 1;
  $name = PSEscape($invoice->{name});
  $address = PSEscape($invoice->{address});
  my $postalcode = $invoice->{postalcode};
  $postalcode =~ s/ //g;
  my $city = PSEscape($invoice->{city});
  my $yourref = PSEscape($invoice->{yourref});
  $attention = PSEscape($attention) || "";
  $payterms = PSEscape($payterms);
  my $frm = "/#copies $copies def\n/frmTYPE 1 def\n";
  $frm .= "/frmDNAME [($attention)] def\n";
  $frm .= "/frmDCOMPANY [($name)] def\n";
  $frm .= "/frmDADDRESS [($address)\n";
  $frm .= "($postalcode $city)\n";
  $frm .= "] def\n";
  $frm .= "/frmDATE ($date) def\n";
  $frm .= "/frmINVOICE ($invoice->{invoicenr}) def\n";
  $frm .= "/frmORDER ($invoice->{ordernr}) def\n";
  $frm .= "/frmTAX (25 \%) def\n";
  $frm .= "/frmOURREF () def
/frmPAYMENTCOND ($payterms) def
/frmYOURREF ($yourref) def
/frmCREDITCODE () def
/frmDELIVERYCODE () def
/frmFREIGHTCODE () def
/frmNOTE [] def
/frmCUSTNOTE [] def
/frmWEIGHT () def\n";

  $frm .= "/frmITEMS [\n";
# Totals
  my ($vat_pct, $prodac_id) = 0;
  my ($totamt, $totalvat, $totalweight);
  my $deliverytype;

# ----- Invoice line loop -----
  while ( $orderline = $ordl->fetchrow_hashref ) {
# Amounts
   my $linevat_id = $vat_id;
   my $linetotal = $orderline->{linetotal};
   my $linevat = $orderline->{linevat};
   my $lineamt = $linetotal - $linevat;
   $totamt += $linetotal;
   $totalvat += $linevat;

# GL update / lines
   if ($orderline->{art_id}) {
    $q = "SELECT v.percent, p.pgrp_id, a.delivery, a.unitweight FROM pd_vat v,
     pd_productgroup p, pd_article a
     WHERE a.art_id = $orderline->{art_id} AND a.pgrp_id = p.pgrp_id AND v.vat_id = p.vat_id";
    $sth = $db->prepare($q);
    $sth->execute;
    ($vat_pct, $prodac_id, $delivery, $unitweight) = $sth->fetchrow();
   };
   $deliverytype = $delivery if $delivery;
   $totalweight += $unitweight*$orderline->{qty};
   $text = 'Fakturering';
   $vat_pct = 0 if !$vat_pct;
   $prodac_id = 0 if !$prodac_id;
   ($batchdr, $batchcr, $journalnr) = updGLbalance($debtac_id, $prodac_id, $custac_id, $periodnr, $linetotal, 0, $text, $glbatchnr, $batchdr, $batchcr, $journalnr);
   ($batchdr, $batchcr, $journalnr) = updGLbalance($salesac_id, $prodac_id, $custac_id, $periodnr, 0, $lineamt, $text, $glbatchnr, $batchdr, $batchcr, $journalnr);
   ($batchdr, $batchcr, $journalnr) = updGLbalance($vatac_id, $prodac_id, $custac_id, $periodnr, 0, $linevat, $text, $glbatchnr, $batchdr, $batchcr, $journalnr);
# PS Stuff / lines
   setlocale(LC_NUMERIC, "da_DK");
   my $desc = PSEscape($orderline->{description});
   my $qty = sprintf "%.0f",$orderline->{qty};
   my $lprice = sprintf "%.2f",$orderline->{unitprice};
   my $lbel = sprintf "%.2f",$orderline->{linetotal};
   my $ltot = sprintf "%.2f",$totamt;
   $frm .= "[($desc) () ($qty) () ($lprice) ($lbel) ($ltot)]\n";
   setlocale(LC_NUMERIC, "en_US");
  }	# ----- Finished lines -----
  insertDelivery($invoice->{ordernr},$deliverytype,$totalweight) if $deliverytype;
  updAR($invoice->{contact_id}, $invoice->{periodnr}, $totamt, $invoice->{invoicenr});
# Finish db
  my $rc = $db->commit;
# Finish PS
  setlocale(LC_NUMERIC, "da_DK");
  my $ltot = sprintf "%.2f",$totamt;
  my $lvat = sprintf "%.2f",$totalvat;
  setlocale(LC_NUMERIC, "en_US");
  $frm .= "] def
/frmVATPERCENT (25\%) def
/frmPRICETOTAL () def
/frmPRICEBEFOREVAT () def
/frmVATAMOUNT ($lvat) def
/frmPRICEAFTERVAT ($ltot) def
/frmVATFREEAMOUNT () def
ShowFormula\n";
  print OUT $frm;
 }
# End
 $q = "UPDATE gl_batch SET totaldr = $batchdr, totalcr = $batchcr WHERE batch_id = $glbatchnr";
# print "->$q\n";
 $sth = $db->do($q);
 my $rc = $db->commit;

 $invh->finish();

 $db->disconnect;
};

sub FormatDate {
 my $type = shift;
 # Date = systemdate
 my $yr     = (localtime)[5]+1900;
 my $mth     = (localtime)[4]+1;
 my $day    = (localtime)[3];
 my $date;
 if ($type) {
  $date =  sprintf "%.2d.%.2d.%4u", ($day, $mth, $yr);
 } else {
  $date = sprintf "%2d/%2d-%4u", ($day, $mth, $yr);
 }
}

sub PSEscape {
 my $v = shift;
 if ($v) {
  $v =~ s/\\/\\\\/;
  $v =~ s/\(/\\\(/;
  $v =~ s/\%/\\\%/;
  $v =~ s/\)/\\\)/;
 };
$v
}

sub findTemplate {
 my ($ordernr, $contact_id, $custgrp_id) = @_;
 if ($ordernr) {
  $q = "SELECT template FROM so_invoicetemplate i, so_invtplchoice c
   WHERE c.ordernr = $ordernr AND c.invtplnr = i.invtplnr;";
  $sth = $db->prepare($q);
  $sth->execute or die("Can't execute $q\n");
  my ($template) = $sth->fetchrow();
 return $template if $template;
 };
 if ($contact_id) {
  $q = "SELECT template FROM so_invoicetemplate i, so_invtplchoice c
   WHERE c.contact_id = $contact_id AND c.invtplnr = i.invtplnr;";
  $sth = $db->prepare($q);
  $sth->execute or die("Can't execute $q\n");
  my ($template) = $sth->fetchrow();
  return $template if $template;
 };
 if ($custgrp_id) {
  $q = "SELECT template FROM so_invoicetemplate i, so_invtplchoice c
   WHERE c.custgrp_id = $custgrp_id AND c.invtplnr = i.invtplnr;";
  $sth = $db->prepare($q);
  $sth->execute or die("Can't execute $q\n");
  my ($template) = $sth->fetchrow();
  return $template if $template;
 };
 $all_template;
}

sub insertDelivery {
 my ($ordernr, $deliverytype,$weight) = @_;
 my $q = "SELECT form,orderstatus FROM pd_deliverytype WHERE type = $deliverytype";
 $sth = $db->prepare($q);
 $sth->execute or return $sth->errstr;
 return if !$sth->rows;
 my ($form,$orderstatus) = $sth->fetchrow();
 $form = $db->quote($form);
 $q = "INSERT INTO so_delivery VALUES ($ordernr,100,$form,'',$weight,'now');";
 $sth = $db->prepare($q);
 $sth->execute or return $sth->errstr;
}

sub updAR {
 my ($contact_id, $periodnr, $amount, $invoicenr) = @_;
# arcontact
 $q = "SELECT 1 FROM ar_contact WHERE contact_id = $contact_id and periodnr = '$periodnr'";
 $sth = $db->prepare($q);
 $sth->execute or return $sth->errstr;
 if (!$sth->rows) {
  $q = "INSERT INTO ar_contact (contact_id, periodnr, sales, paid)
   VALUES ($contact_id, '$periodnr', $amount, 0)";
 } else {
  $q = "UPDATE ar_contact SET sales = sales + '$amount'::decimal, updated='now'
   WHERE contact_id = $contact_id AND periodnr = '$periodnr'";
 };
 $sth = $db->prepare($q);
 $sth->execute or return $sth->errstr;
 $q = "INSERT INTO ar_journal (contact_id, periodnr, invoicenr, total)
  VALUES ($contact_id, '$periodnr', $invoicenr, $amount)";
 $sth = $db->prepare($q);
 $sth->execute or return $sth->errstr;
}

sub updGLbalance {
 my $ag_id;
 my ($coacct_id, $prodacct_id, $custacct_id, $periodnr, $debit, $credit, $text, $glbatchnr, $batchdr, $batchcr, $journalnr) = @_;
 $q = "SELECT gl_posting($coacct_id, $prodacct_id, $custacct_id, '$periodnr',
  $debit, $credit, '$text', $amounttype, $currency_id, $glbatchnr, ".++$journalnr.")";
 $sth = $db->prepare($q);
 $sth->execute or return $sth->errstr;                                          
 $batchdr += $debit;
 $batchcr += $credit;
 return ($batchdr, $batchcr, $journalnr);
}

sub help {
 print "Syntax: ip.pl invoicebatchnr\n";
 exit 1;
}
