-- Project prefix: pj_ systemid: 9
-- Tables for the Project Module
-- Copyright (c) 2001 The FreeMoney Project
-- This work is released under the GPL
-- $Id: create_tables_project.sql,v 1.11 2001/03/31 17:29:37 kaare Exp $

-- Sequences
DROP SEQUENCE pj_project_project_id _id ;
DROP SEQUENCE pj_task_id uence;
DROP SEQUENCE pj_tr_id uence;
DROP SEQUENCE pj_pb_id uence;

CREATE SEQUENCE pj_project_project_id _id  INCREMENT 1;
CREATE SEQUENCE pj_task_id uence    INCREMENT 1;
CREATE SEQUENCE pj_tr_id uence      INCREMENT 1;
CREATE SEQUENCE pj_pb_id uence      INCREMENT 1;

--Tables
DROP TABLE pj_project;
DROP TABLE pj_task;
DROP TABLE pj_timereport;
DROP TABLE pj_projectbudget;

CREATE TABLE pj_project (
       project_id            integer NOT NULL DEFAULT nextval('pj_project_project_id _id '),
       acct_id              integer,
       company_id            integer,
       project_number       varchar(20),
       project_name         varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (project_id ),
 FOREIGN KEY (acct_id )     REFERENCES gl_chartofaccount,
 FOREIGN KEY (company_id )   REFERENCES co_contact
);

COMMENT ON TABLE pj_project IS 'This is the primary table for projects.
See also: table!!pj_task, table!!pj_timerreport and table!!pj_projectbudget.';
COMMENT ON COLUMN pj_project.acct_id  IS 'Account ID of which this project belongs.
See also: table!!gl_chartofaccount.';
COMMENT ON COLUMN pj_project.company_id  IS 'References to the customer.
See also: table!!co_contact.';
COMMENT ON COLUMN pj_project.project_id  IS 'Uniq identifier.';
COMMENT ON COLUMN pj_project.project_number IS 'User defined project number.';
COMMENT ON COLUMN pj_project.project_name IS 'User defined project name.';
COMMENT ON COLUMN pj_project.updated IS 'Time stamp for update.';

CREATE TABLE pj_task (
       task_id              integer NOT NULL DEFAULT nextval('pj_tasksequence'),
       project_id            integer,
       task_name            varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (task_id ),
 FOREIGN KEY (project_id )  REFERENCES pj_project
);

COMMENT ON TABLE task IS 'The task performed on a project.
See also: table!!pj_project, table!!pj_timereport and table!!pj_projectbudget.';

CREATE TABLE pj_timereport (
       tr_id                integer NOT NULL DEFAULT nextval('pj_trsequence'),
       contact_id           integer,
       task_id              integer,
       starttime            timestamp,
       endtime              timestamp,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (tr_id ),
 FOREIGN KEY (contact_id )  REFERENCES co_contact,
 FOREIGN KEY (task_id )     REFERENCES pj_task
);

COMMENT ON TABLE pj_timereport IS 'The time spend on a task in a project.
See also: table!!pj_project, table!!pj_task and table!!pj_projectbudget.';

CREATE TABLE pj_projectbudget (
       pb_id                integer NOT NULL DEFAULT nextval('pj_pbsequence'),
       project_id            integer,
       pb_type              integer,
       time_est             decimal,
       startdate            date,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (pb_id ),
 FOREIGN KEY (project_id )  REFERENCES pj_project
);

-- Views

-- Functions

-- Triggers
